// quick example of how to use the player in your own stuff

#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include <kernel.h>
#include <libspu.h>

#include "xmplay.h"
#include "intro.h" //intro hit file

#define PACKETMAX 10000
#define PACKETMAX2 (PACKETMAX*24)

#define OT_LENGTH 14	  

GsOT WorldOrderingTable[2];
GsOT_TAG zSortTable[2][1<<OT_LENGTH];   
PACKET GpuOutputPacket[2][PACKETMAX2];

extern unsigned char xmData[];
extern unsigned char vhData[];
extern unsigned char vbData[];

/**** Initialise SPU data ****/

#define MALLOC_MAX 128
char spu_malloc_rec [SPU_MALLOC_RECSIZ * (MALLOC_MAX + 1)];

/**** FEEDBACK SCRUCTURES ****/
XM_Feedback Feedback;
XM_Feedback *FB;
XM_HeaderInfo HInfo;
XM_HeaderInfo *Inf;
XM_VABInfo VInfo;
XM_VABInfo *VInf;

//XM Stuff
#define XM_ID1 0
int VABID;
int Song_ID1 =-1;


//Prototyping
void InitSPUHardware(void);

//Main
int main(void)
{
	unsigned int outputBufferIndex;
	int XMHeaderSize;
	u_char *XMHeaderAddr;
	int SongSize;
	u_char *XMAddr;

	SetVideoMode(MODE_PAL);
	ResetGraph(0);
	GsInitGraph(320, 256, GsOFSGPU|GsNONINTER, 0, 0);
	GsDefDispBuff(0, 0, 0, 256);

	WorldOrderingTable[0].length = OT_LENGTH;
	WorldOrderingTable[1].length = OT_LENGTH;
	WorldOrderingTable[0].org = zSortTable[0];
	WorldOrderingTable[1].org = zSortTable[1];
	
	//XM STUFF
	InitSPUHardware();			/* Start up SPU */
	XM_OnceOffInit(XM_NTSC);	/* NTSC/PAL */
	XM_SetStereo();				/* or XM_SetMono for mono output */
	
	//Allocate buffer for the XM Header and set the address
	XMHeaderSize = XM_GetFileHeaderSize();
	XMHeaderAddr = (u_char*)malloc(XMHeaderSize);
	XM_SetFileHeaderAddress(XMHeaderAddr);
	
	//Allocate buffer for the Song and set the address
	SongSize = XM_GetSongSize();
	XMAddr = (u_char*)malloc(SongSize);
	XM_SetSongAddress(XMAddr);
	
	//Init Samples
	VABID = XM_VABInit(vhData, vbData);	
	
	//Init XM
	InitXMData(xmData, XM_ID1, XM_UseXMPanning); /* XM_UseXMPanning/XM_UseS3MPanning */
	
	//Read header data
	Inf = &HInfo;
	XM_GetHeaderInfo(0, Inf);	/* XM_ID,HeaderInfo structure */
	
	//Set callback
	VSyncCallback(XM_Update);
	
	//Start song
	Song_ID1=XM_Init(	/* PLAY A SONG */
		VABID,			/* VAB ID returned from XM_VABInit */
		XM_ID1,		/* XM ID */
		0,					/* SongID (-1 to let the player allocate an ID*/
		0,					/* First Channel to playback on */
		XM_Loop,		/* Looping? */
		-1,				/* Play bitmask (-1 = all channels playing)*/
		XM_Music,	/* Music or SFX?*/
		0					/* Start offset or SFX Pattern */
	);
	
	
	FntLoad(320, 0);	
	FntOpen(8, 0, 312, 248, 0, 512);	

	while(1)
	{
		//Clear display and set workbase
		outputBufferIndex = GsGetActiveBuff();
		GsSetWorkBase((PACKET*)GpuOutputPacket[outputBufferIndex]);
		GsClearOt(0, 0, &WorldOrderingTable[outputBufferIndex]);
		
		//Get XM Feedback
		FB=&Feedback;
		XM_GetFeedback(0,FB);	/* SongID,Feedback structure */
		
		FntPrint(" PSX XM PLAYER EXAMPLE        \n");
		FntPrint(" EXAMPLE BY zeroZshadow\n");
		FntPrint(" XMPLAY LIB BY Jason Page\n");
		FntPrint(" VERSION 1.0\n\n");
		FntPrint("SONGPOS: %3d\nPATTERN: %3d\nPATTERNPOS: %2d\n", Feedback.SongPos, Feedback.CurrentPattern, Feedback.PatternPos);
		FntFlush(-1);
		
		DrawSync(0);
		VSync(0);
		GsSwapDispBuff();
		
		GsSortClear(0, 0, 0, &WorldOrderingTable[outputBufferIndex]); 
		
		GsDrawOt(&WorldOrderingTable[outputBufferIndex]);
	}
	XM_Exit();
	return 0;
}

void InitSPUHardware(void)
{
	/** Do Some SPU Hardware stuff **/
	SpuInit ();
	SpuSetCommonMasterVolume(0x3fff,0x3fff);
	SpuInitMalloc (MALLOC_MAX, spu_malloc_rec);
}


